object AnotherSolution extends App {
  type Set = Int => Boolean
  val bound = 1000
  
  def emptySet: Set = x => false
  
  def add(s: Set, e: Int): Set = x => s(x) || x == e
  def toString(s: Set): String = {
    val xs = for (i <- -bound to bound if s(i)) yield i
    xs.mkString("{", ",", "}")
  }

  def printSet(s: Set) {
    println(toString(s))
  }
  
  def foldLeftSet[T](s: Set, ident: T, f:(T,Int)=>T):T = {
    def helper(i: Int, answer: T): T =
      if (i > bound) answer
      else if (s(i)) helper(i+1,f(answer,i))
      else helper(i+1,answer)
    helper(-bound,ident)
  }
  
  val s1 = add(add(add(emptySet,5),6),7)
  val s2 = add(add(add(add(emptySet,5),6),7),8)
  
  def sum(s: Set): Int = foldLeftSet(s,0,(x:Int,y)=>x+y)
  def product(s: Set): Int = foldLeftSet(s,1,(x:Int,y)=>x*y)
  def size(s: Set): Int = foldLeftSet(s,0,(x:Int,y) => x+1)
  def exists(s: Set, p: Int => Boolean): Boolean = foldLeftSet(s,false,(x:Boolean,y) => x||p(y))
  def forall(s: Set, p: Int => Boolean): Boolean = foldLeftSet(s,true,(x:Boolean,y) => x&&p(y))
  def subset(s: Set, t: Set): Boolean = foldLeftSet(s,true,(x:Boolean,y) => t(y))
  def filter(s: Set, p: Int => Boolean): Set = foldLeftSet(s,emptySet,(x:Set,y)=>if(p(y)) add(x,y) else x)
  def map(s: Set, f: Int => Int): Set = foldLeftSet(s,emptySet,(x:Set,y)=>add(x,f(y)))
                                               
  print("s1=")
  printSet(s1)
  print("s2=")
  printSet(s2)
  println("sum(s1)="+sum(s1))
  println("product(s1)="+product(s1))
  println("size(s1)="+size(s1))
  println("exists(s1,x=>x%2==0)="+exists(s1,x=>x%2==0))
  println("forall(s1,x=>x%2==0)="+forall(s1,x=>x%2==0))
  println("subset(s1,s2)="+subset(s1,s2))
  println("subset(s2,s1)="+subset(s2,s1))
  print("filter(s1,x=>x%2==0)")
  printSet(filter(s1,x=>x%2==0))
  print("filter(s2,x=>x%2==0)")
  printSet(filter(s2,x=>x%2==0))
  print("map(s1,x=>3*x)=")
  printSet(map(s1,x=>3*x))
  print("map(s2,x=>x*x)=")
  printSet(map(s2,x=>x*x))
  
}
