object Sets {
  type Set = Int => Boolean
  val bound = 1000

  // returns true if elem is in s, false otherwise
  def contains(s: Set, elem: Int): Boolean = 
    s(elem)

  // returns a set consisting of a single element, elem
  def singletonSet(elem: Int): Set = 
    x => x == elem

  // returns an empty set
  def emptySet: Set = 
    x => false

  // inserts element x into set s
  def add(s: Set, x: Int): Set = 
    e => contains(s,e) || x == e
    //union(s,singletonSet(x))

  // returns the union of s and t
  def union(s: Set, t: Set): Set = 
    x => contains(s,x) || contains(t,x)

  // returns the intersection of s and t
  def intersect(s: Set, t: Set): Set = 
    x => contains(s,x) && contains(t,x)

  // returns the difference of s and t, s-t
  def difference(s: Set, t: Set): Set = 
    x => contains(s,x) && !contains(t,x)

  // returns true is s is a subset of t, false otherwise
  def subset(s: Set, t: Set): Boolean = {
    def helper(x: Int): Boolean = {
      if (x > bound) 
        true
      else if ((contains(s,x) && !contains(t,x)) || (contains(s,-x) && !contains(t,-x)))
        false
      else
        helper(x+1)
    }
    helper(0)
  }

  // returns true is s is equal to t, false otherwise
  def equal(s: Set, t: Set): Boolean =
    subset(s,t) && subset(t,s)

  // returns the number of elements in s
  def size(s: Set): Int = {
    def helper(x: Int, csize: Int): Int = {
      if (x > bound) 
        csize
      else if ((contains(s,x) && contains(s,-x)))
        helper(x+1,csize+2)
      else if ((contains(s,x) && !contains(s,-x)) || (!contains(s,x) && contains(s,-x)))
        helper(x+1,csize+1)
      else
        helper(x+1,csize)
    }
    helper(0,0)
  }

  // returns a subset of s of elements of s that satisfy the predicate p
  def filter(s: Set, p: Int => Boolean): Set = 
    x => s(x) && p(x)

  // returns true if all elements of s satisfy predicate p
  def forall(s: Set, p: Int => Boolean): Boolean = {
    def helper(x: Int): Boolean = {
      if (x > bound) 
        true
      else if ((contains(s,x) && !p(x)) || (contains(s,-x) && !p(-x)))
        false
      else 
        helper(x+1)
    }
    helper(0)
  }


  // returns true if at least one element of s satisfies predicate p
  def exists(s: Set, p: Int => Boolean): Boolean = {
    def helper(x: Int): Boolean = {
      if (x > bound) 
        false
      else if ((contains(s,x) && p(x)) || (contains(s,-x) && p(-x)))
        true
      else 
        helper(x+1)
    }
    helper(0)
  }

  // returns a new set obtained by applying f to each element of s
  // and forming a set of the results
  // e.g. if s = {1, 2, 3} and f = (x>x*x) the map(s,f) = {1, 4, 9}
  def map(s: Set, f: Int => Int): Set = {
    def helper(x: Int, cset: Set): Set = {
      if (x > bound) 
        cset
      else if (contains(s,x))
        helper(x+1,union(cset,singletonSet(f(x))))
      else if (contains(s,-x))
        helper(x+1,union(cset,singletonSet(f(-x))))
      else 
        helper(x+1,cset)
    }
    helper(0,emptySet)
  }

  def toString(s: Set): String = {
    val xs = for (i <- -bound to bound if contains(s, i)) yield i
    xs.mkString("{", ",", "}")
  }

  def printSet(s: Set) {
    println(toString(s))
  }

}
