object BST {

  abstract class BT
  case class Tree(left: BT, value: Int, right: BT) extends BT
  object Nil extends BT

  def minBST(t: BT): Int = t match {
    case Nil => Int.MinValue
    case Tree(Nil,x,right) => x
    case Tree(left,value,right) => minBST(left)
  }

  def maxBST(t: BT): Int = t match {
    case Nil => Int.MaxValue
    case Tree(left,x,Nil) => x
    case Tree(left,value,right) => maxBST(right)
  }

  def isBST(t: BT): Boolean = t match {
    case Nil => true
    case Tree(left,value,right) =>
      isBST(left) && 
      isBST(right) &&
      (value > minBST(left)) && 
      (value < maxBST(right))
  }

  def memberBST(x: Int, t: BT): Boolean = t match {
    case Nil => false
    case Tree(left,value,right) if (x == value) => true
    case Tree(left,value,right) if (x < value)  => memberBST(x,left)
    case Tree(left,value,right) if (x > value)  => memberBST(x,right)
  }

  def insertBST(x: Int, t: BT): BT = t match {
    case Nil => Tree(Nil,x,Nil)
    case Tree(left,value,right) if (x == value) => t
    case Tree(left,value,right) if (x < value)  => Tree(insertBST(x,left),value,right)
    case Tree(left,value,right) if (x > value)  => Tree(left,value,insertBST(x,right))
  }

  def heightBST(t: BT): Int = t match {
    case Nil => -1
    case Tree(left,value,right) => 1 + scala.math.max(heightBST(left),heightBST(right))
  }

  def deleteBST(x: Int, t: BT): BT = t match {
    case Nil => Nil
    case Tree(left,value,Nil)   if (x == value) => left
    case Tree(left,value,Nil)   if (x != value) => Tree(deleteBST(x,left),value,Nil)
    case Tree(Nil,value,right)  if (x == value) => right
    case Tree(Nil,value,right)  if (x != value) => Tree(Nil,value,deleteBST(x,right))
    case Tree(left,value,right) if (x == value) => Tree(left,minBST(right),deleteBST(minBST(right),right))
    case Tree(left,value,right) if (x < value)  => Tree(deleteBST(x,left),value,right)
    case Tree(left,value,right) if (x > value)  => Tree(left,value,deleteBST(x,right))
  }

  def sizeBST(t: BT): Int = t match {
    case Nil => 0
    case Tree(left,value,right) => 1 + sizeBST(left)+sizeBST(right)
  }

  def toString(t: BT): String = t match {
    case Nil => " "
    case Tree(left,value,right) => 
      toString(left)++value.toString++toString(right)
  }

  def toString2(t: BT): String = t match {
    case Nil => "nil"
    case Tree(left,value,right) => 
      "tree("++toString2(left)++","++value.toString++","++toString2(right)++")"
  }

  def printBST(s: BT) = {
    println(toString2(s))
  }

  def printBSTAsSet(s: BT) = {
    println("("+toString(s)+")")
  }

}
